using System;
using System.Data.SqlClient;
using NHibernate;
using NHibernate.Cfg;
using NHibernate.Tool.hbm2ddl;
using NUnit.Framework;
using System.Linq;

namespace NHibernateSample
{
    [TestFixture]
    public class PersistenceTestFixture
    {
        private static ISessionFactory _sessionFactory;

        [SetUp]
        public void ThisRunsBeforeEverythingElse()
        {
            //prepare configuration (hibernate.cfg.xml used by default)
            var cfg = new Configuration().Configure();

            //create database if not exists, outputting info messages from db
            using (var con = new SqlConnection(cfg.Properties["connection.connection_string"].Replace("ClubTest","master")))
            using (var cmd = new SqlCommand("if not exists ( select 1 from sys.databases where name = 'ClubTest') begin create database ClubTest end;", con))
            {   
                con.Open();
                con.InfoMessage += (sender, args) => Console.WriteLine(args.Message);
                cmd.ExecuteNonQuery();
            }

            //create the tables ClubTest automatically from mapping
            new SchemaExport(cfg).Create(false,true);
            
            //build a session factory from config 
            //for later use
            _sessionFactory = cfg.BuildSessionFactory();
        }

        [Test]
        public void QueryingAndSortingWorks()
        {
            //create a few clubs
            var club1 = new Club { Name = "Club A" };
            var club2 = new Club { Name = "Club B" };
            
            //assign first two users to a club, but not 3rd user
            var user1 = new User {Username = "tobinharris", Club=club2};
            var user2 = new User { Username = "joebloggs", Club = club1 };
            var user3 = new User { Username = "homersimpson" }; //no club

            //open NH session to save users
            using(var session = _sessionFactory.OpenSession())
            using(var tran = session.BeginTransaction())
            {
                //save all our objects
                foreach (var obj in new object[] { club1, club2, user1, user2, user3 })
                    session.Save(obj);

                //commit the transaction
                tran.Commit();
            }

            //open brand new session to issue query
            using(var session = _sessionFactory.OpenSession())
            {
                //find users, ordered by club name. 
                //clubs are eager-fetched to avoid n+1 uneccessary db trips
                var found = session.CreateQuery("select u from u in class User left outer join fetch u.Club as c order by c.Name").List<User>();
                
                //spit out something to the console, so I can eyeball it
                Console.WriteLine("\n\nQuery Results\n=============");
                foreach(var user in found)
                    Console.WriteLine("{0}{1}",user.Username.PadRight(found.Max(u=>u.Username.Length + 3)), user.Club!=null ? user.Club.Name :"-");

                //assert that ordering works as expected, even with the null
                Assert.IsNull(found[0].Club);
                Assert.AreEqual("Club A",found[1].Club.Name);
                Assert.AreEqual("Club B", found[2].Club.Name);
            }
        }
    }
}